import{n as d,q as E,T as h,v as T,w as m,x as f,y as o,E as g,z as N,A as v,B as w,C as y}from"./DjhvlsAc.js";function p(r){var n=document.createElement("template");return n.innerHTML=r.replaceAll("<!>","<!---->"),n.content}function a(r,n){var e=m;e.nodes===null&&(e.nodes={start:r,end:n,a:null,t:null})}function M(r,n){var e=(n&h)!==0,l=(n&T)!==0,t,i=!r.startsWith("<!>");return()=>{if(f)return a(o,null),o;t===void 0&&(t=p(i?r:"<!>"+r),e||(t=d(t)));var s=l||E?document.importNode(t,!0):t.cloneNode(!0);if(e){var _=d(s),u=s.lastChild;a(_,u)}else a(s,s);return s}}function x(r,n,e="svg"){var l=!r.startsWith("<!>"),t=`<${e}>${l?r:"<!>"+r}</${e}>`,i;return()=>{if(f)return a(o,null),o;if(!i){var s=p(t),_=d(s);i=d(_)}var u=i.cloneNode(!0);return a(u,u),u}}function F(r,n){return x(r,n,"svg")}function L(r=""){if(!f){var n=v(r+"");return a(n,n),n}var e=o;return e.nodeType!==w&&(e.before(e=v()),y(e)),a(e,e),e}function O(){if(f)return a(o,null),o;var r=document.createDocumentFragment(),n=document.createComment(""),e=v();return r.append(n,e),a(n,e),r}function P(r,n){if(f){var e=m;((e.f&g)===0||e.nodes.end===null)&&(e.nodes.end=o),N();return}r!==null&&r.before(n)}const A="5";var c;typeof window<"u"&&((c=window.__svelte??(window.__svelte={})).v??(c.v=new Set)).add(A);export{P as a,a as b,O as c,p as d,F as e,M as f,L as t};
